@@ -2,6 +2,7 @@
 import asyncio
 import errno
 import logging
+import mimetypes
 import os
 import socket
 from typing import Tuple, Optional
@@ -12,6 +13,7 @@ from urllib3.exceptions import IncompleteRead
 from fsspec.spec import AbstractBufferedFile
 from fsspec.utils import infer_storage_options, tokenize, setup_logging as setup_logger
 from fsspec.asyn import AsyncFileSystem, sync, sync_wrapper, FSTimeoutError
+from fsspec.callbacks import _DEFAULT_CALLBACK
 
 import aiobotocore
 import botocore
@@ -22,6 +24,14 @@ from botocore.exceptions import ClientError, ParamValidationError
 from s3fs.errors import translate_boto_error
 from s3fs.utils import S3BucketRegionCache, ParamKwargsHelper, _get_brange, FileExpired
 
+# ClientPayloadError can be thrown during an incomplete read. aiohttp is a dependency of
+# aiobotocore, we guard the import here in case this dependency is replaced in a future version
+# of aiobotocore.
+try:
+    from aiohttp import ClientPayloadError
+except ImportError:
+    ClientPayloadError = None
+
 
 logger = logging.getLogger("s3fs")
 
@@ -35,11 +45,33 @@ if "S3FS_LOGGING_LEVEL" in os.environ:
     setup_logging()
 
 
-MANAGED_COPY_THRESHOLD = 5 * 2 ** 30
+MANAGED_COPY_THRESHOLD = 5 * 2**30
 S3_RETRYABLE_ERRORS = (socket.timeout, IncompleteRead)
 
+if ClientPayloadError is not None:
+    S3_RETRYABLE_ERRORS += (ClientPayloadError,)
+
 _VALID_FILE_MODES = {"r", "w", "a", "rb", "wb", "ab"}
 
+_PRESERVE_KWARGS = [
+    "CacheControl",
+    "ContentDisposition",
+    "ContentEncoding",
+    "ContentLanguage",
+    "ContentLength",
+    "ContentType",
+    "Expires",
+    "WebsiteRedirectLocation",
+    "ServerSideEncryption",
+    "SSECustomerAlgorithm",
+    "SSEKMSKeyId",
+    "BucketKeyEnabled",
+    "StorageClass",
+    "ObjectLockMode",
+    "ObjectLockRetainUntilDate",
+    "ObjectLockLegalHoldStatus",
+    "Metadata",
+]
 
 key_acls = {
     "private",
@@ -159,7 +191,7 @@ class S3FileSystem(AsyncFileSystem):
     connect_timeout = 5
     retries = 5
     read_timeout = 15
-    default_block_size = 5 * 2 ** 20
+    default_block_size = 5 * 2**20
     protocol = ["s3", "s3a"]
     _extra_tokenize_attributes = ("default_block_size",)
 
@@ -252,7 +284,7 @@ class S3FileSystem(AsyncFileSystem):
             except S3_RETRYABLE_ERRORS as e:
                 logger.debug("Retryable error: %s", e)
                 err = e
-                await asyncio.sleep(min(1.7 ** i * 0.1, 15))
+                await asyncio.sleep(min(1.7**i * 0.1, 15))
             except Exception as e:
                 logger.debug("Nonretryable error: %s", e)
                 err = e
@@ -481,6 +513,7 @@ class S3FileSystem(AsyncFileSystem):
         cache_type=None,
         autocommit=True,
         requester_pays=None,
+        cache_options=None,
         **kwargs
     ):
         """Open a file for reading or writing
@@ -549,6 +582,7 @@ class S3FileSystem(AsyncFileSystem):
             cache_type=cache_type,
             autocommit=autocommit,
             requester_pays=requester_pays,
+            cache_options=cache_options,
         )
 
     async def _lsdir(
@@ -883,11 +917,11 @@ class S3FileSystem(AsyncFileSystem):
         resp["Body"].close()
         return data
 
-    async def _pipe_file(self, path, data, chunksize=50 * 2 ** 20, **kwargs):
+    async def _pipe_file(self, path, data, chunksize=50 * 2**20, **kwargs):
         bucket, key, _ = self.split_path(path)
         size = len(data)
         # 5 GB is the limit for an S3 PUT
-        if size < min(5 * 2 ** 30, 2 * chunksize):
+        if size < min(5 * 2**30, 2 * chunksize):
             return await self._call_s3(
                 "put_object", Bucket=bucket, Key=key, Body=data, **kwargs
             )
@@ -921,7 +955,9 @@ class S3FileSystem(AsyncFileSystem):
             )
         self.invalidate_cache(path)
 
-    async def _put_file(self, lpath, rpath, chunksize=50 * 2 ** 20, **kwargs):
+    async def _put_file(
+        self, lpath, rpath, callback=_DEFAULT_CALLBACK, chunksize=50 * 2**20, **kwargs
+    ):
         bucket, key, _ = self.split_path(rpath)
         if os.path.isdir(lpath):
             if key:
@@ -930,11 +966,19 @@ class S3FileSystem(AsyncFileSystem):
             else:
                 await self._mkdir(lpath)
         size = os.path.getsize(lpath)
+        callback.set_size(size)
+
+        if "ContentType" not in kwargs:
+            content_type, _ = mimetypes.guess_type(lpath)
+            if content_type is not None:
+                kwargs["ContentType"] = content_type
+
         with open(lpath, "rb") as f0:
-            if size < min(5 * 2 ** 30, 2 * chunksize):
+            if size < min(5 * 2**30, 2 * chunksize):
                 await self._call_s3(
                     "put_object", Bucket=bucket, Key=key, Body=f0, **kwargs
                 )
+                callback.relative_update(size)
             else:
 
                 mpu = await self._call_s3(
@@ -956,6 +1000,7 @@ class S3FileSystem(AsyncFileSystem):
                             Key=key,
                         )
                     )
+                    callback.relative_update(len(chunk))
 
                 parts = [
                     {"PartNumber": i + 1, "ETag": o["ETag"]} for i, o in enumerate(out)
@@ -971,7 +1016,9 @@ class S3FileSystem(AsyncFileSystem):
             self.invalidate_cache(rpath)
             rpath = self._parent(rpath)
 
-    async def _get_file(self, rpath, lpath, version_id=None):
+    async def _get_file(
+        self, rpath, lpath, callback=_DEFAULT_CALLBACK, version_id=None
+    ):
         bucket, key, vers = self.split_path(rpath)
         if os.path.isdir(lpath):
             return
@@ -983,13 +1030,15 @@ class S3FileSystem(AsyncFileSystem):
             **self.req_kw,
         )
         body = resp["Body"]
+        callback.set_size(resp.get("ContentLength", None))
         try:
             with open(lpath, "wb") as f0:
                 while True:
-                    chunk = await body.read(2 ** 16)
+                    chunk = await body.read(2**16)
                     if not chunk:
                         break
-                    f0.write(chunk)
+                    segment_len = f0.write(chunk)
+                    callback.relative_update(segment_len)
         finally:
             body.close()
 
@@ -1023,14 +1072,13 @@ class S3FileSystem(AsyncFileSystem):
                 )
                 return {
                     "ETag": out["ETag"],
-                    "Key": "/".join([bucket, key]),
                     "LastModified": out["LastModified"],
-                    "Size": out["ContentLength"],
                     "size": out["ContentLength"],
                     "name": "/".join([bucket, key]),
                     "type": "file",
-                    "StorageClass": "STANDARD",
+                    "StorageClass": out.get("StorageClass", "STANDARD"),
                     "VersionId": out.get("VersionId"),
+                    "ContentType": out.get("ContentType"),
                 }
             except FileNotFoundError:
                 pass
@@ -1055,10 +1103,8 @@ class S3FileSystem(AsyncFileSystem):
                 or out.get("CommonPrefixes", [])
             ):
                 return {
-                    "Key": "/".join([bucket, key]),
                     "name": "/".join([bucket, key]),
                     "type": "directory",
-                    "Size": 0,
                     "size": 0,
                     "StorageClass": "DIRECTORY",
                 }
@@ -1165,29 +1211,24 @@ class S3FileSystem(AsyncFileSystem):
     async def _metadata(self, path, refresh=False, **kwargs):
         """Return metadata of path.
 
-        Metadata is cached unless `refresh=True`.
-
         Parameters
         ----------
         path : string/bytes
             filename to get metadata for
         refresh : bool (=False)
-            if False, look in local cache for file metadata first
+            (ignored)
         """
         bucket, key, version_id = self.split_path(path)
-        if refresh or path not in self._metadata_cache:
-            response = await self._call_s3(
-                "head_object",
-                kwargs,
-                Bucket=bucket,
-                Key=key,
-                **version_id_kw(version_id),
-                **self.req_kw,
-            )
-            meta = {k.replace("_", "-"): v for k, v in response["Metadata"].items()}
-            self._metadata_cache[path] = meta
-
-        return self._metadata_cache[path]
+        response = await self._call_s3(
+            "head_object",
+            kwargs,
+            Bucket=bucket,
+            Key=key,
+            **version_id_kw(version_id),
+            **self.req_kw,
+        )
+        meta = {k.replace("_", "-"): v for k, v in response["Metadata"].items()}
+        return meta
 
     metadata = sync_wrapper(_metadata)
 
@@ -1492,14 +1533,14 @@ class S3FileSystem(AsyncFileSystem):
         )
         self.invalidate_cache(path2)
 
-    async def _copy_managed(self, path1, path2, size, block=5 * 2 ** 30, **kwargs):
+    async def _copy_managed(self, path1, path2, size, block=5 * 2**30, **kwargs):
         """Copy file between locations on S3 as multi-part
 
         block: int
             The size of the pieces, must be larger than 5MB and at most 5GB.
             Smaller blocks mean more calls, only useful for testing.
         """
-        if block < 5 * 2 ** 20 or block > 5 * 2 ** 30:
+        if block < 5 * 2**20 or block > 5 * 2**30:
             raise ValueError("Copy block size must be 5MB<=block<=5GB")
         bucket, key, version = self.split_path(path2)
         mpu = await self._call_s3(
@@ -1743,15 +1784,15 @@ class S3File(AbstractBufferedFile):
     """
 
     retries = 5
-    part_min = 5 * 2 ** 20
-    part_max = 5 * 2 ** 30
+    part_min = 5 * 2**20
+    part_max = 5 * 2**30
 
     def __init__(
         self,
         s3,
         path,
         mode="rb",
-        block_size=5 * 2 ** 20,
+        block_size=5 * 2**20,
         acl="",
         version_id=None,
         fill_cache=True,
@@ -1759,6 +1800,7 @@ class S3File(AbstractBufferedFile):
         autocommit=True,
         cache_type="bytes",
         requester_pays=False,
+        cache_options=None,
     ):
         bucket, key, path_version_id = s3.split_path(path)
         if not key:
@@ -1775,7 +1817,7 @@ class S3File(AbstractBufferedFile):
         self.s3_additional_kwargs = s3_additional_kwargs or {}
         self.req_kw = {"RequestPayer": "requester"} if requester_pays else {}
         if "r" not in mode:
-            if block_size < 5 * 2 ** 20:
+            if block_size < 5 * 2**20:
                 raise ValueError("Block size must be >=5MB")
         else:
             if version_id and s3.version_aware:
@@ -1790,7 +1832,13 @@ class S3File(AbstractBufferedFile):
                 self.details = s3.info(path)
                 self.version_id = self.details.get("VersionId")
         super().__init__(
-            s3, path, mode, block_size, autocommit=autocommit, cache_type=cache_type
+            s3,
+            path,
+            mode,
+            block_size,
+            autocommit=autocommit,
+            cache_type=cache_type,
+            cache_options=cache_options,
         )
         self.s3 = self.fs  # compatibility
 
@@ -1798,15 +1846,39 @@ class S3File(AbstractBufferedFile):
         self.append_block = False
 
         if "a" in mode and s3.exists(path):
-            loc = s3.info(path)["size"]
-            if loc < 5 * 2 ** 20:
+            # See:
+            # put: https://boto3.amazonaws.com/v1/documentation/api/latest
+            # /reference/services/s3.html#S3.Client.put_object
+            #
+            # head: https://boto3.amazonaws.com/v1/documentation/api/latest
+            # /reference/services/s3.html#S3.Client.head_object
+            head = self._call_s3(
+                "head_object",
+                self.kwargs,
+                Bucket=bucket,
+                Key=key,
+                **version_id_kw(version_id),
+                **self.req_kw,
+            )
+
+            head = {
+                key: value
+                for key, value in head.items()
+                if key in _PRESERVE_KWARGS and key not in self.s3_additional_kwargs
+            }
+
+            loc = head.pop("ContentLength")
+            if loc < 5 * 2**20:
                 # existing file too small for multi-upload: download
                 self.write(self.fs.cat(self.path))
             else:
                 self.append_block = True
             self.loc = loc
 
-        if "r" in mode:
+            # Reflect head
+            self.s3_additional_kwargs.update(head)
+
+        if "r" in mode and "ETag" in self.details:
             self.req_kw["IfMatch"] = self.details["ETag"]
 
     def _call_s3(self, method, *kwarglist, **kwargs):
@@ -1868,7 +1940,7 @@ class S3File(AbstractBufferedFile):
         """
         if self.writable():
             raise NotImplementedError(
-                "cannot update metadata while file " "is open for writing"
+                "cannot update metadata while file is open for writing"
             )
         return self.fs.setxattr(self.path, copy_kwargs=copy_kwargs, **kwargs)
 
@@ -1891,7 +1963,7 @@ class S3File(AbstractBufferedFile):
         except OSError as ex:
             if ex.args[0] == errno.EINVAL and "pre-conditions" in ex.args[1]:
                 raise FileExpired(
-                    filename=self.details["name"], e_tag=self.details["ETag"]
+                    filename=self.details["name"], e_tag=self.details.get("ETag")
                 ) from ex
             else:
                 raise
@@ -1963,6 +2035,7 @@ class S3File(AbstractBufferedFile):
                     Key=self.key,
                     Bucket=self.bucket,
                     Body=data,
+                    ACL=self.acl,
                     **self.kwargs,
                 )
             else:
