@@ -15,11 +15,14 @@ import pytest
 import moto
 from itertools import chain
 import fsspec.core
+from dateutil.tz import tzutc
+
 import s3fs.core
 from s3fs.core import S3FileSystem
 from s3fs.utils import ignoring, SSEParams
 from botocore.exceptions import NoCredentialsError
 from fsspec.asyn import sync
+from fsspec.callbacks import Callback
 from packaging import version
 
 test_bucket_name = "test"
@@ -161,7 +164,7 @@ def expect_errno(expected_errno):
 
 
 def test_simple(s3):
-    data = b"a" * (10 * 2 ** 20)
+    data = b"a" * (10 * 2**20)
 
     with s3.open(a, "wb") as f:
         f.write(data)
@@ -174,7 +177,7 @@ def test_simple(s3):
 
 @pytest.mark.parametrize("default_cache_type", ["none", "bytes", "mmap"])
 def test_default_cache_type(s3, default_cache_type):
-    data = b"a" * (10 * 2 ** 20)
+    data = b"a" * (10 * 2**20)
     s3 = S3FileSystem(
         anon=False,
         default_cache_type=default_cache_type,
@@ -256,6 +259,9 @@ def test_info(s3):
     linfo = s3.ls(a, detail=True)[0]
     assert abs(info.pop("LastModified") - linfo.pop("LastModified")).seconds < 1
     info.pop("VersionId")
+    info.pop("ContentType")
+    linfo.pop("Key")
+    linfo.pop("Size")
     assert info == linfo
     parent = a.rsplit("/", 1)[0]
     s3.invalidate_cache()  # remove full path from the cache
@@ -663,7 +669,7 @@ def test_bulk_delete(s3):
     assert not s3.exists(test_bucket_name + "/nested/nested2/file1")
 
 
-@pytest.mark.xfail(reason="anon user is still priviliged on moto")
+@pytest.mark.xfail(reason="anon user is still privileged on moto")
 def test_anonymous_access(s3):
     with ignoring(NoCredentialsError):
         s3 = S3FileSystem(anon=True, client_kwargs={"endpoint_url": endpoint_uri})
@@ -694,6 +700,22 @@ def test_s3_file_info(s3):
         s3.info(fn + "another")
 
 
+def test_content_type_is_set(s3, tmpdir):
+    test_file = str(tmpdir) + "/test.json"
+    destination = test_bucket_name + "/test.json"
+    open(test_file, "w").write("text")
+    s3.put(test_file, destination)
+    assert s3.info(destination)["ContentType"] == "application/json"
+
+
+def test_content_type_is_not_overrided(s3, tmpdir):
+    test_file = os.path.join(str(tmpdir), "test.json")
+    destination = os.path.join(test_bucket_name, "test.json")
+    open(test_file, "w").write("text")
+    s3.put(test_file, destination, ContentType="text/css")
+    assert s3.info(destination)["ContentType"] == "text/css"
+
+
 def test_bucket_exists(s3):
     assert s3.exists(test_bucket_name)
     assert not s3.exists(test_bucket_name + "x")
@@ -841,16 +863,16 @@ def test_copy(s3):
 
 
 def test_copy_managed(s3):
-    data = b"abc" * 12 * 2 ** 20
+    data = b"abc" * 12 * 2**20
     fn = test_bucket_name + "/test/biggerfile"
     with s3.open(fn, "wb") as f:
         f.write(data)
-    sync(s3.loop, s3._copy_managed, fn, fn + "2", size=len(data), block=5 * 2 ** 20)
+    sync(s3.loop, s3._copy_managed, fn, fn + "2", size=len(data), block=5 * 2**20)
     assert s3.cat(fn) == s3.cat(fn + "2")
     with pytest.raises(ValueError):
-        sync(s3.loop, s3._copy_managed, fn, fn + "3", size=len(data), block=4 * 2 ** 20)
+        sync(s3.loop, s3._copy_managed, fn, fn + "3", size=len(data), block=4 * 2**20)
     with pytest.raises(ValueError):
-        sync(s3.loop, s3._copy_managed, fn, fn + "3", size=len(data), block=6 * 2 ** 30)
+        sync(s3.loop, s3._copy_managed, fn, fn + "3", size=len(data), block=6 * 2**30)
 
 
 @pytest.mark.parametrize("recursive", [True, False])
@@ -877,7 +899,7 @@ def test_get_put(s3, tmpdir):
 
 def test_get_put_big(s3, tmpdir):
     test_file = str(tmpdir.join("test"))
-    data = b"1234567890A" * 2 ** 20
+    data = b"1234567890A" * 2**20
     open(test_file, "wb").write(data)
 
     s3.put(test_file, test_bucket_name + "/bigfile")
@@ -886,7 +908,46 @@ def test_get_put_big(s3, tmpdir):
     assert open(test_file, "rb").read() == data
 
 
-@pytest.mark.parametrize("size", [2 ** 10, 2 ** 20, 10 * 2 ** 20])
+def test_get_put_with_callback(s3, tmpdir):
+    test_file = str(tmpdir.join("test.json"))
+
+    class BranchingCallback(Callback):
+        def branch(self, path_1, path_2, kwargs):
+            kwargs["callback"] = BranchingCallback()
+
+    cb = BranchingCallback()
+    s3.get(test_bucket_name + "/test/accounts.1.json", test_file, callback=cb)
+    assert cb.size == 1
+    assert cb.value == 1
+
+    cb = BranchingCallback()
+    s3.put(test_file, test_bucket_name + "/temp", callback=cb)
+    assert cb.size == 1
+    assert cb.value == 1
+
+
+def test_get_file_with_callback(s3, tmpdir):
+    test_file = str(tmpdir.join("test.json"))
+
+    cb = Callback()
+    s3.get_file(test_bucket_name + "/test/accounts.1.json", test_file, callback=cb)
+    assert cb.size == os.stat(test_file).st_size
+    assert cb.value == cb.size
+
+
+@pytest.mark.parametrize("size", [2**10, 10 * 2**20])
+def test_put_file_with_callback(s3, tmpdir, size):
+    test_file = str(tmpdir.join("test.json"))
+    with open(test_file, "wb") as f:
+        f.write(b"1234567890A" * size)
+
+    cb = Callback()
+    s3.put_file(test_file, test_bucket_name + "/temp", callback=cb)
+    assert cb.size == os.stat(test_file).st_size
+    assert cb.value == cb.size
+
+
+@pytest.mark.parametrize("size", [2**10, 2**20, 10 * 2**20])
 def test_pipe_cat_big(s3, size):
     data = b"1234567890A" * size
     s3.pipe(test_bucket_name + "/bigfile", data)
@@ -940,10 +1001,10 @@ def test_errors_cause_preservings(monkeypatch, s3):
 
     assert type(exc.value.__cause__).__name__ == "NoSuchBucket"
 
-    async def list_objects_v2(*args, **kwargs):
+    async def head_object(*args, **kwargs):
         raise NoCredentialsError
 
-    monkeypatch.setattr(type(s3.s3), "list_objects_v2", list_objects_v2)
+    monkeypatch.setattr(type(s3.s3), "head_object", head_object)
 
     # Since the error is not translate, the __cause__ would
     # be None
@@ -1030,12 +1091,34 @@ def test_write_small(s3):
         f.write(b"hello")
     assert s3.cat(test_bucket_name + "/test") == b"hello"
     s3.open(test_bucket_name + "/test", "wb").close()
-    assert s3.info(test_bucket_name + "/test")["Size"] == 0
+    assert s3.info(test_bucket_name + "/test")["size"] == 0
+
+
+def test_write_small_with_acl(s3):
+    bucket, key = (test_bucket_name, "test-acl")
+    filename = bucket + "/" + key
+    body = b"hello"
+    public_read_acl = {
+        "Permission": "READ",
+        "Grantee": {
+            "URI": "http://acs.amazonaws.com/groups/global/AllUsers",
+            "Type": "Group",
+        },
+    }
+
+    with s3.open(filename, "wb", acl="public-read") as f:
+        f.write(body)
+    assert s3.cat(filename) == body
+
+    assert (
+        public_read_acl
+        in sync(s3.loop, s3.s3.get_object_acl, Bucket=bucket, Key=key)["Grants"]
+    )
 
 
 def test_write_large(s3):
     "flush() chunks buffer when processing large singular payload"
-    mb = 2 ** 20
+    mb = 2**20
     payload_size = int(2.5 * 5 * mb)
     payload = b"0" * payload_size
 
@@ -1043,12 +1126,12 @@ def test_write_large(s3):
         fd.write(payload)
 
     assert s3.cat(test_bucket_name + "/test") == payload
-    assert s3.info(test_bucket_name + "/test")["Size"] == payload_size
+    assert s3.info(test_bucket_name + "/test")["size"] == payload_size
 
 
 def test_write_limit(s3):
     "flush() respects part_max when processing large singular payload"
-    mb = 2 ** 20
+    mb = 2**20
     block_size = 15 * mb
     payload_size = 44 * mb
     payload = b"0" * payload_size
@@ -1058,7 +1141,7 @@ def test_write_limit(s3):
 
     assert s3.cat(test_bucket_name + "/test") == payload
 
-    assert s3.info(test_bucket_name + "/test")["Size"] == payload_size
+    assert s3.info(test_bucket_name + "/test")["size"] == payload_size
 
 
 def test_write_small_secure(s3):
@@ -1082,9 +1165,9 @@ def test_write_large_secure(s3):
     s3.mkdir("mybucket")
 
     with s3.open("mybucket/myfile", "wb") as f:
-        f.write(b"hello hello" * 10 ** 6)
+        f.write(b"hello hello" * 10**6)
 
-    assert s3.cat("mybucket/myfile") == b"hello hello" * 10 ** 6
+    assert s3.cat("mybucket/myfile") == b"hello hello" * 10**6
 
 
 def test_write_fails(s3):
@@ -1103,21 +1186,21 @@ def test_write_fails(s3):
 
 def test_write_blocks(s3):
     with s3.open(test_bucket_name + "/temp", "wb") as f:
-        f.write(b"a" * 2 * 2 ** 20)
-        assert f.buffer.tell() == 2 * 2 ** 20
+        f.write(b"a" * 2 * 2**20)
+        assert f.buffer.tell() == 2 * 2**20
         assert not (f.parts)
         f.flush()
-        assert f.buffer.tell() == 2 * 2 ** 20
+        assert f.buffer.tell() == 2 * 2**20
         assert not (f.parts)
-        f.write(b"a" * 2 * 2 ** 20)
-        f.write(b"a" * 2 * 2 ** 20)
+        f.write(b"a" * 2 * 2**20)
+        f.write(b"a" * 2 * 2**20)
         assert f.mpu
         assert f.parts
-    assert s3.info(test_bucket_name + "/temp")["Size"] == 6 * 2 ** 20
-    with s3.open(test_bucket_name + "/temp", "wb", block_size=10 * 2 ** 20) as f:
-        f.write(b"a" * 15 * 2 ** 20)
+    assert s3.info(test_bucket_name + "/temp")["size"] == 6 * 2**20
+    with s3.open(test_bucket_name + "/temp", "wb", block_size=10 * 2**20) as f:
+        f.write(b"a" * 15 * 2**20)
         assert f.buffer.tell() == 0
-    assert s3.info(test_bucket_name + "/temp")["Size"] == 15 * 2 ** 20
+    assert s3.info(test_bucket_name + "/temp")["size"] == 15 * 2**20
 
 
 def test_readline(s3):
@@ -1141,7 +1224,7 @@ def test_readline_empty(s3):
 
 
 def test_readline_blocksize(s3):
-    data = b"ab\n" + b"a" * (10 * 2 ** 20) + b"\nab"
+    data = b"ab\n" + b"a" * (10 * 2**20) + b"\nab"
     with s3.open(a, "wb") as f:
         f.write(data)
     with s3.open(a, "rb") as f:
@@ -1150,7 +1233,7 @@ def test_readline_blocksize(s3):
         assert result == expected
 
         result = f.readline()
-        expected = b"a" * (10 * 2 ** 20) + b"\n"
+        expected = b"a" * (10 * 2**20) + b"\n"
         assert result == expected
 
         result = f.readline()
@@ -1212,12 +1295,12 @@ def test_writable(s3):
 
 def test_merge(s3):
     with s3.open(a, "wb") as f:
-        f.write(b"a" * 10 * 2 ** 20)
+        f.write(b"a" * 10 * 2**20)
 
     with s3.open(b, "wb") as f:
-        f.write(b"a" * 10 * 2 ** 20)
+        f.write(b"a" * 10 * 2**20)
     s3.merge(test_bucket_name + "/joined", [a, b])
-    assert s3.info(test_bucket_name + "/joined")["Size"] == 2 * 10 * 2 ** 20
+    assert s3.info(test_bucket_name + "/joined")["size"] == 2 * 10 * 2**20
 
 
 def test_append(s3):
@@ -1230,24 +1313,54 @@ def test_append(s3):
     assert s3.cat(test_bucket_name + "/nested/file1") == data + b"extra"
 
     with s3.open(a, "wb") as f:
-        f.write(b"a" * 10 * 2 ** 20)
+        f.write(b"a" * 10 * 2**20)
     with s3.open(a, "ab") as f:
         pass  # append, no write, big file
-    assert s3.cat(a) == b"a" * 10 * 2 ** 20
+    assert s3.cat(a) == b"a" * 10 * 2**20
 
     with s3.open(a, "ab") as f:
         assert f.parts is None
         f._initiate_upload()
         assert f.parts
-        assert f.tell() == 10 * 2 ** 20
+        assert f.tell() == 10 * 2**20
         f.write(b"extra")  # append, small write, big file
-    assert s3.cat(a) == b"a" * 10 * 2 ** 20 + b"extra"
+    assert s3.cat(a) == b"a" * 10 * 2**20 + b"extra"
 
     with s3.open(a, "ab") as f:
-        assert f.tell() == 10 * 2 ** 20 + 5
-        f.write(b"b" * 10 * 2 ** 20)  # append, big write, big file
-        assert f.tell() == 20 * 2 ** 20 + 5
-    assert s3.cat(a) == b"a" * 10 * 2 ** 20 + b"extra" + b"b" * 10 * 2 ** 20
+        assert f.tell() == 10 * 2**20 + 5
+        f.write(b"b" * 10 * 2**20)  # append, big write, big file
+        assert f.tell() == 20 * 2**20 + 5
+    assert s3.cat(a) == b"a" * 10 * 2**20 + b"extra" + b"b" * 10 * 2**20
+
+    # Keep Head Metadata
+    head = dict(
+        CacheControl="public",
+        ContentDisposition="string",
+        ContentEncoding="gzip",
+        ContentLanguage="ru-RU",
+        ContentType="text/csv",
+        Expires=datetime.datetime(2015, 1, 1, 0, 0, tzinfo=tzutc()),
+        Metadata={"string": "string"},
+        ServerSideEncryption="AES256",
+        StorageClass="REDUCED_REDUNDANCY",
+        WebsiteRedirectLocation="https://www.example.com/",
+        BucketKeyEnabled=False,
+    )
+    with s3.open(a, "wb", **head) as f:
+        f.write(b"data")
+
+    with s3.open(a, "ab") as f:
+        f.write(b"other")
+
+    with s3.open(a) as f:
+        filehead = {
+            k: v
+            for k, v in f._call_s3(
+                "head_object", f.kwargs, Bucket=f.bucket, Key=f.key
+            ).items()
+            if k in head
+        }
+        assert filehead == head
 
 
 def test_bigger_than_block_read(s3):
@@ -1285,7 +1398,6 @@ def _get_s3_id(s3):
     return id(s3.s3)
 
 
-@pytest.mark.skipif(sys.version_info[:2] < (3, 7), reason="ctx method only >py37")
 @pytest.mark.parametrize(
     "method",
     [
@@ -1294,7 +1406,7 @@ def _get_s3_id(s3):
             "forkserver",
             marks=pytest.mark.skipif(
                 sys.platform.startswith("win"),
-                reason="'forserver' not available on windows",
+                reason="'forkserver' not available on windows",
             ),
         ),
     ],
@@ -1340,14 +1452,14 @@ def test_upload_with_s3fs_prefix(s3):
     path = "s3://test/prefix/key"
 
     with s3.open(path, "wb") as f:
-        f.write(b"a" * (10 * 2 ** 20))
+        f.write(b"a" * (10 * 2**20))
 
     with s3.open(path, "ab") as f:
-        f.write(b"b" * (10 * 2 ** 20))
+        f.write(b"b" * (10 * 2**20))
 
 
 def test_multipart_upload_blocksize(s3):
-    blocksize = 5 * (2 ** 20)
+    blocksize = 5 * (2**20)
     expected_parts = 3
 
     s3f = s3.open(a, "wb", block_size=blocksize)
@@ -1561,26 +1673,26 @@ def test_change_defaults_only_subsequent():
         S3FileSystem.cachable = False  # don't reuse instances with same pars
 
         fs_default = S3FileSystem(client_kwargs={"endpoint_url": endpoint_uri})
-        assert fs_default.default_block_size == 5 * (1024 ** 2)
+        assert fs_default.default_block_size == 5 * (1024**2)
 
         fs_overridden = S3FileSystem(
-            default_block_size=64 * (1024 ** 2),
+            default_block_size=64 * (1024**2),
             client_kwargs={"endpoint_url": endpoint_uri},
         )
-        assert fs_overridden.default_block_size == 64 * (1024 ** 2)
+        assert fs_overridden.default_block_size == 64 * (1024**2)
 
         # Suppose I want all subsequent file systems to have a block size of 1 GiB
         # instead of 5 MiB:
-        S3FileSystem.default_block_size = 1024 ** 3
+        S3FileSystem.default_block_size = 1024**3
 
         fs_big = S3FileSystem(client_kwargs={"endpoint_url": endpoint_uri})
-        assert fs_big.default_block_size == 1024 ** 3
+        assert fs_big.default_block_size == 1024**3
 
         # Test the other file systems created to see if their block sizes changed
-        assert fs_overridden.default_block_size == 64 * (1024 ** 2)
-        assert fs_default.default_block_size == 5 * (1024 ** 2)
+        assert fs_overridden.default_block_size == 64 * (1024**2)
+        assert fs_default.default_block_size == 5 * (1024**2)
     finally:
-        S3FileSystem.default_block_size = 5 * (1024 ** 2)
+        S3FileSystem.default_block_size = 5 * (1024**2)
         S3FileSystem.cachable = True
 
 
@@ -1818,7 +1930,6 @@ def test_modified(s3):
         s3.modified(path=test_bucket_name)
 
 
-@pytest.mark.skipif(sys.version_info < (3, 7), reason="no asyncio.run in py36")
 def test_async_s3(s3):
     async def _():
         s3 = S3FileSystem(
@@ -1865,7 +1976,6 @@ def test_cat_ranges(s3):
     assert s3.cat_file(fn, start=-5) == data[-5:]
 
 
-@pytest.mark.skipif(sys.version_info < (3, 7), reason="no asyncio.run in py36")
 def test_async_s3_old(s3):
     async def _():
         s3 = S3FileSystem(
@@ -1922,7 +2032,6 @@ def test_with_xzarr(s3):
     nana.to_dataset().to_zarr(store=s3store, mode="w", consolidated=True, compute=True)
 
 
-@pytest.mark.skipif(sys.version_info < (3, 7), reason="no asyncio.run in py36")
 def test_async_close():
     async def _():
         loop = asyncio.get_event_loop()
@@ -2073,12 +2182,12 @@ def test_raise_exception_when_file_has_changed_during_reading(s3):
 def test_s3fs_etag_preserving_multipart_copy(monkeypatch, s3):
     # Set this to a lower value so that we can actually
     # test this without creating giant objects in memory
-    monkeypatch.setattr(s3fs.core, "MANAGED_COPY_THRESHOLD", 5 * 2 ** 20)
+    monkeypatch.setattr(s3fs.core, "MANAGED_COPY_THRESHOLD", 5 * 2**20)
 
     test_file1 = test_bucket_name + "/test/multipart-upload.txt"
     test_file2 = test_bucket_name + "/test/multipart-upload-copy.txt"
 
-    with s3.open(test_file1, "wb", block_size=5 * 2 ** 21) as stream:
+    with s3.open(test_file1, "wb", block_size=5 * 2**21) as stream:
         for _ in range(5):
             stream.write(b"b" * (stream.blocksize + random.randrange(200)))
 
@@ -2102,7 +2211,6 @@ def test_s3fs_etag_preserving_multipart_copy(monkeypatch, s3):
     s3.rm(test_file1)
 
 
-@pytest.mark.skipif(sys.version_info < (3, 7), reason="no asyncio.run in py36")
 def test_sync_from_wihin_async(s3):
     # if treating as sync but within an even loop, e.g., calling from jupyter;
     # IO happens on dedicated thread.
@@ -2123,47 +2231,32 @@ def test_token_paths(s3):
 
 
 def test_same_name_but_no_exact(s3):
-    s3.touch(test_bucket_name + "/very/similiar/prefix1")
-    s3.touch(test_bucket_name + "/very/similiar/prefix2")
-    s3.touch(test_bucket_name + "/very/similiar/prefix3/something")
-    assert not s3.exists(test_bucket_name + "/very/similiar/prefix")
-    assert not s3.exists(test_bucket_name + "/very/similiar/prefi")
-    assert not s3.exists(test_bucket_name + "/very/similiar/pref")
+    s3.touch(test_bucket_name + "/very/similar/prefix1")
+    s3.touch(test_bucket_name + "/very/similar/prefix2")
+    s3.touch(test_bucket_name + "/very/similar/prefix3/something")
+    assert not s3.exists(test_bucket_name + "/very/similar/prefix")
+    assert not s3.exists(test_bucket_name + "/very/similar/prefi")
+    assert not s3.exists(test_bucket_name + "/very/similar/pref")
 
-    assert s3.exists(test_bucket_name + "/very/similiar/")
-    assert s3.exists(test_bucket_name + "/very/similiar/prefix1")
-    assert s3.exists(test_bucket_name + "/very/similiar/prefix2")
-    assert s3.exists(test_bucket_name + "/very/similiar/prefix3")
-    assert s3.exists(test_bucket_name + "/very/similiar/prefix3/")
-    assert s3.exists(test_bucket_name + "/very/similiar/prefix3/something")
+    assert s3.exists(test_bucket_name + "/very/similar/")
+    assert s3.exists(test_bucket_name + "/very/similar/prefix1")
+    assert s3.exists(test_bucket_name + "/very/similar/prefix2")
+    assert s3.exists(test_bucket_name + "/very/similar/prefix3")
+    assert s3.exists(test_bucket_name + "/very/similar/prefix3/")
+    assert s3.exists(test_bucket_name + "/very/similar/prefix3/something")
 
-    assert not s3.exists(test_bucket_name + "/very/similiar/prefix3/some")
+    assert not s3.exists(test_bucket_name + "/very/similar/prefix3/some")
 
-    s3.touch(test_bucket_name + "/starting/very/similiar/prefix")
+    s3.touch(test_bucket_name + "/starting/very/similar/prefix")
 
-    assert not s3.exists(test_bucket_name + "/starting/very/similiar/prefix1")
-    assert not s3.exists(test_bucket_name + "/starting/very/similiar/prefix2")
-    assert not s3.exists(test_bucket_name + "/starting/very/similiar/prefix3")
-    assert not s3.exists(test_bucket_name + "/starting/very/similiar/prefix3/")
-    assert not s3.exists(test_bucket_name + "/starting/very/similiar/prefix3/something")
+    assert not s3.exists(test_bucket_name + "/starting/very/similar/prefix1")
+    assert not s3.exists(test_bucket_name + "/starting/very/similar/prefix2")
+    assert not s3.exists(test_bucket_name + "/starting/very/similar/prefix3")
+    assert not s3.exists(test_bucket_name + "/starting/very/similar/prefix3/")
+    assert not s3.exists(test_bucket_name + "/starting/very/similar/prefix3/something")
 
-    assert s3.exists(test_bucket_name + "/starting/very/similiar/prefix")
-    assert s3.exists(test_bucket_name + "/starting/very/similiar/prefix/")
-
-
-def test_info_with_permission_error_for_list_objects(monkeypatch, s3):
-    s3.touch(test_bucket_name + "/very/similiar/prefix")
-
-    async def list_objects_v2(*args, **kwargs):
-        if kwargs.pop("Prefix").endswith("/"):
-            return {}
-        else:
-            raise PermissionError
-
-    monkeypatch.setattr(type(s3.s3), "list_objects_v2", list_objects_v2)
-    assert not s3.exists(test_bucket_name + "/very/similiar/prefix1")
-    assert s3.exists(test_bucket_name + "/very/similiar/prefix")
-    assert s3.info(test_bucket_name + "/very/similiar/prefix")["type"] == "file"
+    assert s3.exists(test_bucket_name + "/starting/very/similar/prefix")
+    assert s3.exists(test_bucket_name + "/starting/very/similar/prefix/")
 
 
 def test_leading_forward_slash(s3):
@@ -2174,7 +2267,7 @@ def test_leading_forward_slash(s3):
 
 
 def test_lsdir(s3):
-    # https://github.com/dask/s3fs/issues/475
+    # https://github.com/fsspec/s3fs/issues/475
     s3.find(test_bucket_name)
 
     d = test_bucket_name + "/test"
